#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Do something for uninstalling the hradius package."""

import os

import ionas.request as request
from fal_info import (FAL_SERVICECTRL_PATH, Results, ReqTypes)

from pkgaddon.hradius_config import (RADIUS_SERVICE,)

import manualctrl


SERVICELIST_NAME = "servicelist"
SERVICE_NAME = "service"
NAME_NAME = "name"

SERVICECTRL_REQ_XML = """<servicectrl>
    <servicelist>
        <service><name>%s</name></service>
    </servicelist>
</servicectrl>
""" % (RADIUS_SERVICE,)


def __send_request(target, type, body=""):
    """Send request entry to "target"."""

    entry = request.Entry(type, body)
    return request.Request(target).request(entry).get_entry()


def stop_radius_service():
    """Stop service for the radius by a event handler."""

    __send_request(FAL_SERVICECTRL_PATH, ReqTypes.STOP, SERVICECTRL_REQ_XML)


def delete_radius_service():
    """Delete service for the radius package from a servicectrl's configuration
       file.
    """

    try:
        servicectrl_config = \
                    __send_request(FAL_SERVICECTRL_PATH, ReqTypes.LOAD).get()

        servicelist = servicectrl_config.get_node(SERVICELIST_NAME)
        for (idx, service) in \
                enumerate(servicelist.get_node_list(SERVICE_NAME)):

            try:
                if service.get_node(NAME_NAME).get_text() == RADIUS_SERVICE:
                    servicelist.delete_child(SERVICE_NAME, index=idx)
                    __send_request(FAL_SERVICECTRL_PATH, ReqTypes.SAVE,
                                    servicectrl_config)
                    return
            except:
                pass
    except:
        pass


#
# Main
#

stop_radius_service()
delete_radius_service()
manualctrl.uninstall()
