# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import division

import os
import re

from django.utils.translation import ugettext as _
from django import forms
from django.http import HttpResponseRedirect, HttpResponse
from project.view import FuncView
from . import fields
from ..radius import Radius


class CreateForm(forms.Form):

    def __init__(self, **kwargs):
        kwargs['prefix'] = kwargs.get('prefix', 'create')
        super(CreateForm, self).__init__(**kwargs)

    name = fields.UsernameField()


class UpdateForm(CreateForm):

    def __init__(self, **kwargs):
        kwargs['prefix'] = kwargs.get('prefix', 'update')
        super(UpdateForm, self).__init__(**kwargs)

    pk = forms.CharField(widget=forms.widgets.HiddenInput)


class DeleteForm(forms.Form):

    def __init__(self, **kwargs):
        kwargs['prefix'] = kwargs.get('prefix', 'delete')
        super(DeleteForm, self).__init__(**kwargs)

    pk = forms.CharField(label=u"ID", widget=forms.widgets.HiddenInput, required=False)


class ListView(FuncView):

    views = ['create', 'delete', 'download_sv', 'download_cl', 'asc', 'dsc']

    radius = Radius()

    def _get_entries(self):
        return self.radius.get_certs()


    def view(self, request):
        create_form = CreateForm()
        update_form = UpdateForm()
        delete_form = DeleteForm()

        self.radius.load()

        template = os.path.join(self.TEMPLATES, 'entries.html')
        dictionary = {
            'entries': self._get_entries(),
            'create_form': create_form,
            'update_form': update_form,
            'delete_form': delete_form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }


    def view_asc(self, request):
        self.radius.set_sort_asc()
        return self.view(request)


    def view_dsc(self, request):
        self.radius.set_sort_dsc()
        return self.view(request)


    def view_create(self, request):
        create_form = CreateForm(data=request.POST)
        update_form = UpdateForm()
        delete_form = DeleteForm()
        form = create_form

        with self.processing() as p:
            p.break_if_state_change()
            if form.is_valid():
                name = form.cleaned_data['name']
                passphrase = ""

                ret = self.radius.append_cert(name, passphrase)
                if ret == Radius.Ok:
                    form = create_form = CreateForm()
                    self.set_message(_(u'append %(target)s is sunccessful') % {'target': name})
                    p.result(True)
                elif ret == Radius.ErrExistAlready:
                    self.set_message(_(u'%(target)s exist already') % {'target': name})
                    p.result(False)
                else:
                    self.set_message(_(u'append %(target)s is failed') % {'target': name})
                    p.result(False)

        template = os.path.join(self.TEMPLATES, 'entries.html')
        dictionary = {
            'entries': self._get_entries(),
            'create_form': create_form,
            'update_form': update_form,
            'delete_form': delete_form,
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }


    def view_delete(self, request):
        create_form = CreateForm()
        update_form = UpdateForm()
        delete_form = DeleteForm(data=request.POST)
        form = delete_form

        with self.processing() as p:
            p.break_if_state_change()
            if form.is_valid():
                ret = Radius.Failed
                for k, v in request.POST.items():
                    if v == 'on':
                        if self.radius.delete_cert(int(k)) == Radius.Ok:
                            ret = Radius.Ok
                if ret == Radius.Ok:
                    form = delete_form = DeleteForm()
                    self.set_message(_(u'delete is sunccessful'))
                    p.result(True)
                else:
                    self.set_message(_(u'delete is failed'))
                    p.result(False)

        template = os.path.join(self.TEMPLATES, 'entries.html')
        dictionary = {
            'entries': self._get_entries(),
            'create_form': create_form,
            'update_form': update_form,
            'delete_form': delete_form,
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }


    def view_download_sv(self, request):
        try:
            filepath = self.radius.get_path_server_cert()
            filename = os.path.basename(filepath)

            response = HttpResponse(
                open(filepath, 'rb').read(),
                mimetype="application/octet-stream")
            response['Content-Disposition'] = 'attachment; filename=%s' % filename
            return {self.KEY_RESPONSE: response}
        except:
            pass
        return self.view(request)


    def view_download_cl(self, request):
        try:
            pk = request.POST['pk']
            if pk:
                filepath = self.radius.get_path_client_cert(int(pk))
                filename = os.path.basename(filepath)

                response = HttpResponse(
                    open(filepath, 'rb').read(),
                    mimetype="application/octet-stream")
                response['Content-Disposition'] = 'attachment; filename=%s' % filename
                return {self.KEY_RESPONSE: response}
        except:
            pass
        return self.view(request)
