# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import division

import os

from django.utils.translation import ugettext as _
from django import forms
from project.view import FuncView
from . import fields
from ..radius import Radius


class CreateForm(forms.Form):

    def __init__(self, **kwargs):
        kwargs['prefix'] = kwargs.get('prefix', 'create')
        super(CreateForm, self).__init__(**kwargs)

    name = fields.UsernameField()
    password = fields.PasswordField()


class UpdateForm(CreateForm):

    def __init__(self, **kwargs):
        kwargs['prefix'] = kwargs.get('prefix', 'update')
        super(UpdateForm, self).__init__(**kwargs)

    pk = forms.CharField(widget=forms.widgets.HiddenInput)


class DeleteForm(forms.Form):

    def __init__(self, **kwargs):
        kwargs['prefix'] = kwargs.get('prefix', 'delete')
        super(DeleteForm, self).__init__(**kwargs)

    pk = forms.CharField(label=u"ID", widget=forms.widgets.HiddenInput, required=False)  # FIXME


class ListView(FuncView):

    views = ['create', 'update', 'delete', 'asc', 'dsc']
    radius = Radius()


    def _get_entries(self):
        return self.radius.get_users()


    def view(self, request):
        create_form = CreateForm()
        update_form = UpdateForm()
        delete_form = DeleteForm()

        self.radius.load()

        template = os.path.join(self.TEMPLATES, 'entries.html')
        dictionary = {
            'entries': self._get_entries(),
            'create_form': create_form,
            'update_form': update_form,
            'delete_form': delete_form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }


    def view_asc(self, request):
        self.radius.set_sort_asc()
        return self.view(request)


    def view_dsc(self, request):
        self.radius.set_sort_dsc()
        return self.view(request)


    def view_create(self, request):
        create_form = CreateForm(data=request.POST)
        update_form = UpdateForm()
        delete_form = DeleteForm()
        form = create_form

        with self.processing() as p:
            p.break_if_state_change()
            if form.is_valid():
                name = form.cleaned_data['name']
                password = form.cleaned_data['password']

                ret = self.radius.append_user(name, password, False)
                if ret == Radius.Ok:
                    self.radius.save()
                    form = create_form = CreateForm()
                    self.set_message(_(u'append %(target)s is sunccessful') % {'target': name})
                    p.result(True)
                elif ret == Radius.ErrExistAlready:
                    self.set_message(_(u'%(target)s exist already') % {'target': name})
                    p.result(False)
                elif ret == Radius.ErrBadFormat:
                    self.set_message(_(u'invalid user name input format'))
                    p.result(False)
                elif ret == Radius.ErrExceededUpperLimit:
                    self.set_message(_(u'exceeded the upper limit of user'))
                    p.result(False)
                else:
                    self.set_message(_(u'append %(target)s is failed') % {'target': name})
                    p.result(False)

        template = os.path.join(self.TEMPLATES, 'entries.html')
        dictionary = {
            'entries': self._get_entries(),
            'create_form': create_form,
            'update_form': update_form,
            'delete_form': delete_form,
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }


    def view_update(self, request):
        create_form = CreateForm()
        update_form = UpdateForm(data=request.POST)
        delete_form = DeleteForm()
        form = update_form

        with self.processing() as p:
            p.break_if_state_change()
            if form.is_valid():
                name = form.cleaned_data['name']
                password = form.cleaned_data['password']
                ret = self.radius.update_user(name, password)
                if ret == Radius.Ok:
                    self.radius.save()
                    form = update_form = UpdateForm()
                    self.set_message(_(u'update %(target)s is sunccessful') % {'target': name})
                    p.result(True)
                else:
                    self.set_message(_(u'update %(target)s is failed') % {'target': name})
                    p.result(False)

        template = os.path.join(self.TEMPLATES, 'entries.html')
        dictionary = {
            'entries': self._get_entries(),
            'create_form': create_form,
            'update_form': update_form,
            'delete_form': delete_form,
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }


    def view_delete(self, request):
        create_form = CreateForm()
        update_form = UpdateForm()
        delete_form = DeleteForm(data=request.POST)
        form = delete_form

        with self.processing() as p:
            p.break_if_state_change()
            if form.is_valid():
                ret = Radius.Failed
                for k, v in request.POST.items():
                    if v == 'on':
                        if self.radius.delete_user(int(k)) == Radius.Ok:
                            ret = Radius.Ok
                if ret == Radius.Ok:
                    self.radius.save()
                    form = delete_form = DeleteForm()
                    self.set_message(_(u'delete is sunccessful'))
                    p.result(True)
                else:
                    self.set_message(_(u'delete is failed'))
                    p.result(False)

        template = os.path.join(self.TEMPLATES, 'entries.html')
        dictionary = {
            'entries': self._get_entries(),
            'create_form': create_form,
            'update_form': update_form,
            'delete_form': delete_form,
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }
